#ifndef __GLOBAL_DATA_H__
#define __GLOBAL_DATA_H__

#include "tcp_link.h"

#include <QString>
#include <QFont>

#include <QGSettings>
#include <QObject>

#define THEME_GSETTINGS "org.ukui.style"
#define APPLY_FONT_KEY "system-font"
#define APPLY_HIGHT_LIGHT_COLOR "theme-color"

class GlobalData : public QObject
{
    Q_OBJECT
public:
    ~GlobalData();

    void getSystemTheme();
    void getSystemFontSize();
    void getSystemTransparency();

    QFont getFontSize12px();
    QFont getFontSize14px();
    QFont getFontSize16px();
    QFont getFontSize18px();

    /* 本机信息 */
    QString m_uuid;
    QString m_tcpListenIP;
    QString m_tcpListenPort;
    QString m_udpListenPort;

    /* tcp链接表 */
    TcpLink m_tcpLink;

    /* 字体监控 */
    double m_font14pxToPt;
    double m_font12pxToPt; /* 好友列表 信息字体 */
    double m_font16pxToPt; /* 标题栏设置弹窗描述字体 */
    double m_font18pxToPt; /* 聊天框 头像字体  , 主界面昵称*/

    QString m_fontName;
    QString m_colorName;

private:
    GlobalData();

    QGSettings *m_fontGsetting = nullptr;

public:
    static GlobalData *getInstance();

private:
    void init(void);
    void getTcpListenIp(void);
    void getTcpListenPort(void);
    void getUdpListenPort(void);
    void getUuid(void);

    void getFontName();
    void changeFontName();

    void changeFontSize();

signals:
    void signalChangeFont();
    void signalChangeFontSize();
    void signalChangeHightColor();
};

#endif
